/************************************************************************************************************
// LightTracking Class ver 2.5 erstellt am 14. Juni 2007
//
// Diese Klasse kann Lichter im Webcam erkennen und Tracken. Als Ergebnis erhaltet ein Bereichsrechteck (Array), in dem sich
// die Lichter befinden.
//
// Author: Florian Weil --- http://www.derhess.de
//
// Die Klasse kann frei fr nicht kommerzielle Zwecke verwendet werden. 
// Bei kommerzieller Verwendung bitte eine kurze Mail an info@derhess.de 
//

/************************************************************************************************************/

import flash.display.BitmapData;
import flash.filters.BitmapFilter;
import flash.filters.ColorMatrixFilter;
import flash.geom.Rectangle;
import flash.geom.Matrix;
import flash.geom.Point;
import flash.geom.ColorTransform;

import mx.events.EventDispatcher; 

class de.derhess.iaCam.LightTracking {
	// Statische Variablen fuer den Modus zum Tracking
	static public var ONE_LIGHT = 0;
	static public var MORE_LIGHTS = 1;


	// Konfigurationsvariablen
	private var scaleFactor:Number; 
	private var schwellenwert:Number; // Ab welchen Helligkeitswert wird getrackt
	private var actionLimit:Number; // Gibt an wieviel Durchlufe das Tracking erfloglos sein darf 
	
	// sonstige Members
	private var webcamMovieClip:MovieClip;
	private var webcamBitmap:BitmapData;
	private var output:BitmapData;
	private var action:Number;  // Toleranz, wie oft hintereinander kein Licht gefunden werden darf
	
	//////////////// Speicherung von Ergebnissen ////////////////////
	private var oldResultTracking:Array;  // Speicherung von mehreren LichterTrackings
	
	// Eventhandling
	private var dispatchEvent:Function; 
	public var addEventListener:Function; 
	public var removeEventListener:Function; 
	
	
	// Konstruktor
	function LightTracking(webcamSignal:MovieClip,schwellHelligkeit:Number) 
	{
		EventDispatcher.initialize(this);
		
		var standardSize:Number = 80;
		
		this.scaleFactor = Math.round(100/(webcamSignal._width  / standardSize));  // Perfomance -> Skalierung der Analysebildes auf 80x60px
		
		this.webcamMovieClip = webcamSignal;
		this.webcamBitmap =	new BitmapData(this.webcamMovieClip._width/(100/this.scaleFactor),this.webcamMovieClip._height/(100/this.scaleFactor),true,0xFF000000);
		this.output =	new BitmapData(this.webcamMovieClip._width/(100/this.scaleFactor),this.webcamMovieClip._height/(100/this.scaleFactor),true,0xFF000000);
		
		this.action = 0; // Anfangswert fr Frameanzahl unaktive Lichtquellen
		this.actionLimit = 5;
		
		this.oldResultTracking= new Array();
		// berprfung der Helligkeitsschwellenangabe
		if(schwellHelligkeit > 255) {
			this.schwellenwert = 255;
		} else if (schwellHelligkeit < 0) {
			this.schwellenwert = 0;
		} else {
			this.schwellenwert = schwellHelligkeit;
		}
		
		
		
		// Mach das Bild SW fr die Helligkeitsanalyse --- evtl. spter in der Bildverarbeitung
		var filterMatrix:Array = [0.33,0.33,0.33,0,0.33,
									0.33,0.33,0.33,0,0.33,
									0.33,0.33,0.33,0,0.33,
									1,1,1,1,0.33];
			
		var farbFilter = new ColorMatrixFilter(filterMatrix);
		//webcamSignal.filters = [farbFilter]; 
		this.webcamBitmap.applyFilter(this.webcamBitmap,this.webcamBitmap.rectangle,new Point(0,0),farbFilter);
		
		
	} // Ende Konstruktor
	
	////////////////// //////////////////////////////////////////////////////////
	//////////// Initialisierung Tracking
	public function startTracking(modus:Number):Void {
		if(modus == 0) {
			// einLicht Tracking
			oneLightTracking();
		} else {
			// Mehrere Lichter tracken
			moreLightsTracking();
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// ///////////Tracking fr eine Lichtquelle   //////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	private function oneLightTracking ()
	{	
		// Schauen ob helle Punkte im Bild vorhanden sind - mindestens 5Pixel 
		var helligkeit:Number = (this.webcamBitmap.height * this.webcamBitmap.width) - this.bildOptimierung();
		
		if(helligkeit >= 1) {
			
			var trackSuccess:Boolean = false;
			// Bilder nach Lichter durchsuchen
			for(var y:Number = 2; y < this.webcamBitmap.height-2; y = y+2) {
				for(var x:Number = 2; x < this.webcamBitmap.width-2;x = x +2) {
				
					//Wenn der Pixel weiss ist dann berprfe die Nachbarn
					if(check_Nachbarn(x,y)) {
						this.oldResultTracking[0] = this.checkLightSize(x,y);
						this.action = 0;
						trackSuccess = true;
						this.dispatchEvent({type:"onTrackingResult", target:this.oldResultTracking[0]}); 
						//return this.oldResultTracking[0];
					}
					
				} // Ender erste Schleife
			} // Ende 2. Schleife
			
			// Wenn kein Licht gefunden wurde, Setze den Zhler hoch, oder werde ein Event, dass kein Licht mehr vorhanden ist
			if(!trackSuccess) { 
				if(this.action < this.actionLimit) {
					this.action++;
				} else {
					this.dispatchEvent({type:"onNoLightFound", target:this}); 
				}
			}
			
		} else if(this.action < this.actionLimit) {
			this.action++;
			// Wenn kein Licht gefunden wurde, gib erstmal ein leeres Rechteck zurck
			this.oldResultTracking[0] = new Rectangle(0,0,0,0);
			this.dispatchEvent({type:"onTrackingFailure", target:this.oldResultTracking[0]}); 
		}  else {
			// Wenn kein Licht gefunden wurde
			this.dispatchEvent({type:"onNoLightFound", target:this}); 
		}
		
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// ///////////Tracking fr mehrere Lichtquellen   //////////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	private function moreLightsTracking()
	{
		var neuResultTracking:Array = new Array();
		var resultCounter:Number = 0;
		
		// Schauen ob helle Punkte im Bild vorhanden sind - mindestens 5Pixel 
		var helligkeit:Number = (this.webcamBitmap.height * this.webcamBitmap.width) - this.bildOptimierung();
		
		if(helligkeit >= 1) {
			
			for(var y:Number = 2; y < this.webcamBitmap.height-2; y = y+2) {
				for(var x:Number = 2; x < this.webcamBitmap.width-2;x = x +2) {
					// berprfen ob der Pixel berprft werden soll oder nicht
					var pixelfarbe = this.webcamBitmap.getPixel32(x,y).toString(16);
					var alpha:Number = Number(pixelfarbe >> 24 & 0xFF);
					
					if(alpha == 255) {
						// Wenn der Pixel weiss ist dann berprfe die Nachbarn
						if(this.check_Nachbarn(x,y)) {	
							neuResultTracking.push(this.checkLightSize(x,y));
						} 
					} // Ende Tracking
				}
			} // Ende Bilddurchlauf
			
			// Wenn schon Lichter vorhanden sind
			if(this.oldResultTracking.length  == neuResultTracking.length) {
				//////////////////////////////// Sotierung der mehreren Lichtquellen ///////
				// Vergleiche die alten Tracking Ergebnisse mit den Neuen
				var ergebnisTracking:Array = new Array
				
				for(var i:Number = 0; i < neuResultTracking.length; i++) {
					var track_index:Number = 0;
					var min_distanz:Number = Math.floor(Point.distance(this.oldResultTracking[0].topLeft,neuResultTracking[i].topLeft));
					
					// suche Passendes Rechteck
					for(var k:Number = 1; k < this.oldResultTracking.length; k++) {
						if(min_distanz > Math.floor(Point.distance(this.oldResultTracking[k].topLeft,neuResultTracking[i].topLeft))) {
							track_index = k;
						}
						
					}
					ergebnisTracking[track_index] = neuResultTracking[i];
				}
				this.oldResultTracking = ergebnisTracking;
				this.dispatchEvent({type:"onTrackingResult", target:this.oldResultTracking}); 
			// Wenn vorher keine Lichter oder neue Lichter vorhanden sind, dann nimm die aktuelle Anzahl
			// ohne deren Beziehung zueinander zu prfen
			} else if(neuResultTracking.length > 0){
				// Ermittlung wieviele Lichter hinzugekommen sind oder verschwunden sind
				var diff:Number = this.oldResultTracking.length - neuResultTracking.length
				if(diff > 0) {
					this.dispatchEvent({type:"onRemoveLight", target:diff}); 
				} else {
					diff = Math.abs(diff);
					this.dispatchEvent({type:"onAddLight", target:diff});
				}
				
				this.oldResultTracking = neuResultTracking;
				this.dispatchEvent({type:"onTrackingResult", target:this.oldResultTracking}); 
			} else if(this.action < this.actionLimit) {
				this.action++;
				// Wenn kein Licht gefunden wurde, gib erstmal ein leeres Rechteck zurck
				this.oldResultTracking = new Array();
				this.dispatchEvent({type:"onTrackingFailure", target:this.oldResultTracking}); 
			}  else {
				this.oldResultTracking = new Array();
				// Wenn kein Licht gefunden wurde und ader Aktiv Status berschritten wurde
				this.dispatchEvent({type:"onNoLightFound", target:this.oldResultTracking}); 
			}
		
		
		}// Ende IF-Block, wenn Lichter vorhanden sind
		else if(this.action < this.actionLimit) {
				this.action++;
				// Wenn kein Licht gefunden wurde, gib erstmal ein leeres Rechteck zurck
				this.oldResultTracking = new Array();
				this.dispatchEvent({type:"onTrackingFailure", target:this.oldResultTracking}); 
		}  else {
				this.oldResultTracking = new Array();
				// Wenn kein Licht gefunden wurde und ader Aktiv Status berschritten wurde
				this.dispatchEvent({type:"onNoLightFound", target:this.oldResultTracking}); 
		}
	} // Ende Funktion
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////7
	//////////////////// Getter und Setter Funktionen
	public function getBitmapData():BitmapData
	{
		return this.output;
	}
	
	// Getter Schwellenwert im Bereich 0 - 255
	public function getThreshold():Number 
	{
		return this.schwellenwert;
	}
	
	// Setter Schwellenwert, aber welcher Helligkeit getrackt werden soll
	public function setThreshold(schwellHelligkeit:Number):Void 
	{
		if(schwellHelligkeit > 255) {
			this.schwellenwert = 255;
		} else if (schwellHelligkeit < 0) {
			this.schwellenwert = 0;
		} else {
			this.schwellenwert = schwellHelligkeit;
		}
	}
	
	
	//////////////////////////////////////////////////////////////////////////
	//////////////// Hilfsfunktionen
	
	// bereitet das Webcambild fr die Analyse auf - erstellt das Binrbild
	private function bildOptimierung():Number
	{
		
		var scaleMatrix:Matrix = new Matrix();
		scaleMatrix.scale(this.scaleFactor/100,this.scaleFactor/100);

		this.webcamBitmap.draw(this.webcamMovieClip,scaleMatrix);
		
		var HexSchwellenwert = "0xFF"+ this.schwellenwert.toString(16) + this.schwellenwert.toString(16) + this.schwellenwert.toString(16);
		var unaktivfarbe:Number = 0xFF000000; // schwarz
		var aktivfarbe:Number = 0xFF00FF00;  // weiss
		
		var changePixel = this.webcamBitmap.threshold(this.webcamBitmap,this.webcamBitmap.rectangle,new Point(0,0),"<=",Number(HexSchwellenwert),unaktivfarbe,aktivfarbe,true);
		this.output.copyPixels(this.webcamBitmap, this.webcamBitmap.rectangle,new Point(0,0));
		
		return changePixel;
	}
	
	// Suche nach mehreren weissen Pixeln unter Beachtung eines bestimmten Rasters -- Perfomancelastig --
	private function check_Nachbarn (x:Number,y:Number):Boolean 
	{
		var counter = 0;
		// oben Mitte
		//if(this.webcamBitmap.getPixel(x,y-1).toString(16) != 0x000000) counter++;
		// rechts oben
		if(this.webcamBitmap.getPixel(x+1,y-1).toString(16) != 0x000000) counter++;
		// rechts Mitte
		if(this.webcamBitmap.getPixel(x+1,y).toString(16) != 0x000000) counter++;
		// rechts unten
		if(this.webcamBitmap.getPixel(x+1,y+1).toString(16) != 0x000000) counter++;
		// unten Mitte
		//if(this.webcamBitmap.getPixel(x,y+1).toString(16) != 0x000000) counter++;
		
		if(counter == 3) {
			return true;
		} else {
			return false;
		}
	}
	
	
	// Wenn ein Licht gefunden wurde, dann ermittle die Gre des Lichtes
	private function checkLightSize(x:Number,y:Number):Rectangle
	{
		var x_track:Number = 0;
		var y_track:Number = 0;
		var width_track:Number = 1;
		var height_track:Number = 1;
		
		x_track = x;
		y_track = y;
		
		// Scanne die Breite des Lichtes
		
		// Scanne die Hoehe des Lichtes funktioniert ganz gut
		while(this.check_Nachbarn(x,y+height_track))  { height_track++; }
		// Scanne obere Breite
		while(this.check_Nachbarn(x + width_track,y)) { width_track++; }
		// Scanne die Breite des Lichtes, aber in deren mittleren Hhe
		y  = Math.floor(y + (height_track / 2));
		var mittlerebreite:Number = 1;
		while(this.webcamBitmap.getPixel(x + mittlerebreite,y).toString(16) != 0x000000) { mittlerebreite++; }
		y = y_track; // Wiederherstellung der alten Koordinate, damit die Schleife noch richtig arbeiten kann
		
		
		// zu kleine Rechteck Abweichung ausgleichen - hauptschlich in der Breite gab es Probleme
		var offset:Number = Math.abs(mittlerebreite - width_track);
		x_track -= Math.floor(offset/2); // Standardwert ist einfach 2
		y_track -= 2; // Standardwert ist einfach 2
		width_track += (offset); // Standardwert ist einfach width_track += 4 oder (offset*2)
		height_track += 4; // Standardwert ist einfach 4
		
		// Ausgabe Rechteck Grsse
		
		this.webcamBitmap.fillRect(new Rectangle(x_track - 4,y_track - 4,width_track+8,height_track+8),0x00000000);
		// Speichern des Ergebnisses
		var scale:Number = 100 /this.scaleFactor;
		var hotpoint:Rectangle = new Rectangle(x_track*scale,y_track*scale,width_track*scale,height_track*scale);
		return hotpoint;
	}


}