/************************************************************************************************************
//CamButtonManager Class ver 1.5 erstellt am 9. Oktober 2007 - portiert auf ActionScript 3
//
// Diese Klasse bearbeitet das Webcambild und verwaltet die Funktionen der CamButtons. Die Klasse berprft ob ein
// ein Button aktiv ist oder nicht und fhrt die dementsprechende Ereignisfunktion aus
//
// Author: Florian Weil --- http://www.derhess.de
//
// Die Klasse kann frei fr nicht kommerzielle Zwecke verwendet werden. 
// Bei kommerzieller Verwendung bitte eine kurze Mail an info@derhess.de 
//
/************************************************************************************************************/

package de.derhess.iaCam {
	
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.filters.BlurFilter;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;

	import de.derhess.iaCam.CamButton;
	import de.derhess.iaCam.CamButtonEvent;
	
	
	public class CamButtonManager {
		
		// Statische Variablen fuer den Modus der Bewegungserkennung
		public static const STATIC:int = 0;
		public static const DYNAMIC:int = 1;
		
		//Konfigurationsmembers
		private var mode:int // Speichern des Analysemodus - STATIC oder DYNAMIC
		private var accuracy:int // Zahl in Prozent bis ein Ereigniss im Button ausgelst wird
		
		// Enthlt das WebcamSignal
		private var webcamContainer:Sprite;
		private var scaleFactor:Number // gibt an um welchen Wert das Bilder herunterskaliert wird -> Perfomance
		
		// Speicherung der Bilddaten fr die Bildanalyse
		private var snapshot:BitmapData;
		private var output:BitmapData;
		
		// Zwischenspeicherung von Bilddaten fr dynamischen Modus
		private var difference:BitmapData;
		private var oldFrame:BitmapData
		
		private var webcamFrames:Array;
		private var quantityFrames:int;
		// Verwaltung aller Buttonliste
		private var buttonlist:Array;
		
		
		// Konstruktor
		public function CamButtonManager(webcamSignal:Sprite,camMode:int) {
			init(webcamSignal,camMode);
		}
		
		// init Funktion aus Perfomancegrnden fr den Konstruktor ausgelagert
		private function init(webcamSignal:Sprite,camMode:int):void {
			
			this.webcamContainer = webcamSignal;
			this.accuracy = 80; // Standardwert 80
			this.mode = camMode;
			this.quantityFrames = 5; // Standardwert 5
			
			var standardSize:Number = 80;
			// Perfomance -> Skalierung der Analysebildes auf 80x60px
			this.scaleFactor = Math.round(100/(this.webcamContainer.width  / standardSize));  
			
			this.webcamFrames = new Array();
			this.buttonlist = new Array();
			this.output =	new BitmapData(this.webcamContainer.width/(100/this.scaleFactor),this.webcamContainer.height/(100/this.scaleFactor),true,0xFF000000);
			this.snapshot = new BitmapData(this.webcamContainer.width/(100/this.scaleFactor),this.webcamContainer.height/(100/this.scaleFactor),true,0xFF000000);
			this.difference = new BitmapData(this.webcamContainer.width/(100/this.scaleFactor),this.webcamContainer.height/(100/this.scaleFactor),true,0xFF000000);
			this.oldFrame = new BitmapData(this.webcamContainer.width/(100/this.scaleFactor),this.webcamContainer.height/(100/this.scaleFactor),true,0xFF000000);
		}
		
		
		// Buttons hinzufegen
		public function createCamButton(xpos:int,ypos:int,width:int,height:int):CamButton {
			var pushed:Number = this.buttonlist.push(new CamButton(xpos,ypos,width,height,this.scaleFactor));
			return this.buttonlist[pushed-1];
		}
		
		// Button der Liste hinzufgen
		public function addCamButton(button:CamButton):Boolean {
			//Suche den Button und wenn schon vorhanden gibt false zurck
			for(var i:Number = 0; i < this.buttonlist.length; i++) {
				// evtl. mit den "==" Operator arbeiten
				if(this.buttonlist[i] === button) {
					return false;
				}
			}
			
			// Wenn der Button nicht vorhanden in der Liste, dann fge diesen hinzu
			this.buttonlist.push(button);
			return true;
		}
		
		// Buttons lschen
		public function removeCamButton(button:CamButton):Boolean {
			//Suche den Button und lsche diesen
			for(var i:Number = 0; i < this.buttonlist.length; i++) {
				// evtl. mit den "==" Operator arbeiten
				if(this.buttonlist[i] === button) {
					this.buttonlist.splice(i,1);
					return true;
				}
			}
			return false;
		}
		
		
		// Erstellen eines Anfangsframes fr den statischen Analysemodus
		public function doSnapshot():void {
			
			var scaleMatrix:Matrix = new Matrix();
			scaleMatrix.scale(this.scaleFactor/100,this.scaleFactor/100);

			this.snapshot.draw(this.webcamContainer,scaleMatrix);
			
		}	
		
		// Starten der ButtonAnalyze
		public function startAnalyze():void {
			if(this.mode == 0) {
				staticImageProcessing();
			} else {
				dynamicImageProcessing();
			}
		
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////// GETTER und SETTER Funktionen
		public function getBitmapData():BitmapData {
				return this.output;
		}
		
		public function getMode():int {
			return this.mode;
		}
		
		// Zahl zwischen 0 und 1 - siehe static Variablen der Klasse
		public function setMode(modus:int):void {
			if(modus == 0)
				this.mode = 0;
			else
				this.mode = 1;
		}
		
		// Wert ab welcher Genauigkeit (in Prozent) der Button als aktiv ausgelst wird
		public function getAccuracy():int {
			return this.accuracy;
		}
		// Setzen des Wertes (in Prozent) ab welcher Genauigkeit der Button als aktiv ausgelst wird
		public function setAccuracy(value:int):void {
			if(value > 0 && value < 100) {
				this.accuracy = value;
			} else if(value < 0) {
				this.accuracy = 0;
				throw new RangeError("Value must be in the range 0-100.");
			} else {
				this.accuracy = 100;
				throw new RangeError("Value must be in the range 0-100.");
			}
		}
		
		
		public function getQuantityFrames():int {
			return this.quantityFrames;
		}
		
		public function setQuantityFrames(number:int):void {
			this.quantityFrames = number;
		}
		
		
		/*///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		///////////////////////////////////////// Hilfsfunktionen //////////////////////////////////////////////////////////////////
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////*/
		
		// extrahiert den User aus dem Bild  - erstellt das Binrbild
		private function staticImageProcessing():void
		{
			var scaleMatrix:Matrix = new Matrix();
			scaleMatrix.scale(this.scaleFactor/100,this.scaleFactor/100);
			
			this.output.draw(this.webcamContainer,scaleMatrix);
			// Differenzverfahren anwenden
			this.output.draw(this.snapshot,new Matrix(),null,"difference");
			
			// Strungen abschwchen -- hier gibt es einige Mglichkeiten die Extrahierung zu verbessern
			var blurX:int = 2;
			var blurY:int = 2;
			var quality:int = 1;
			var filter:BlurFilter = new BlurFilter(blurX, blurY, quality);
			
			this.output.applyFilter(this.output,this.output.rect,new Point(0,0),filter);
			
			// erster Farbwert 0xFF111111 ist der Schwellwert - Hier gibt es auch noch einige zustzliche Mglichkeiten die Extrahierung zu verbessern
			this.output.threshold(this.output,this.output.rect,this.output.rect.topLeft,">",0xFF222222,0xFFFFFFFF,0x00FFFFFF,true);
			//trace(this.output.getPixel(5,5).toString(16));
			// Nach Interaktion im Bild suchen
			this.analyzeButtons();
		}
		
		private function dynamicImageProcessing():void
		{
			//Prinzip siehe MotionDetection Tutorial von Guy Watson - www.flashguru.co.uk
			var scaleMatrix:Matrix = new Matrix();
			scaleMatrix.scale(this.scaleFactor/100,this.scaleFactor/100);
			this.snapshot.draw(webcamContainer,scaleMatrix);
			//copy the current freezeframe
			this.difference =this.snapshot.clone();
			//now draw the previous freezeframe ("BEFORE"!) on top of the current frameframe and apply the 'difference' blendmode
			this.difference.draw(this.oldFrame,new Matrix(),null,"difference");
			//filter out all pixels in the differenced bitmap that are greater than just over black (0xFF111111) and make them white (0xFFFFFFFF)
			//over-write the previous contents of the bitmap
			this.difference.threshold(this.difference,this.difference.rect,this.difference.rect.topLeft,">",0xFF222222,0xFFFFFFFF,0x00FFFFFF,false);
			//add the finished bitmap to a fading image cue
			this.webcamFrames.push(this.difference.clone());
			//if the cue is now greater than the maximum length we specified
			if(this.webcamFrames.length > this.quantityFrames)
			{
				//then remove the bitmap at the very front of the cue
				//and free the memory it was using
				while(webcamFrames.length > this.quantityFrames) this.webcamFrames.shift().dispose();
			}
			
			//clear the out bitmap object so we can start a fresh
			this.output.fillRect(this.output.rect,0xFF000000);
			//copy the current bitmap and save it for the next time this function is called
			this.oldFrame = this.snapshot.clone();
			//loop through each bitmap in the fading image cue
			for(var i:int=0;i<this.webcamFrames.length;++i) {
				//copy all the pixels that are green in the current item in the fading image cue and make them into the degradated green color
				//add those pixels on top of the bitmap that will displayed to the user
				this.output.threshold(this.webcamFrames[i],this.output.rect,this.output.rect.topLeft,"==",0xFFFFFFFF,0xFFFFFFFF,0x00FFFFFF,false);
		
			}
			
			// Strungen abschwchen -- hier gibt es einige Mglichkeiten die Extrahierung zu verbessern
			var blurX:int = 4;
			var blurY:int = 4;
			var quality:int = 1;
			var filter:BlurFilter = new BlurFilter(blurX, blurY, quality);
			
			this.output.applyFilter(this.output,this.output.rect,new Point(0,0),filter);
			//this.output.threshold(this.output,this.output.rectangle,this.output.rectangle.topLeft,"==",0xFFFFFFFF,0xFFFFFFFF,0x00FFFFFF,false);
			// Nach Interaktion im Bild suchen
			this.analyzeButtons();
		}
		
		
		// Hier wird geschaut ob die Buttons aktiv sind. Dementsprechend werden die Ereignisse geworfen
		private function analyzeButtons():void {
			// Gehe Alle CamButtons durch und fhre die Ereignisse aus
			for(var i:int = 0; i < this.buttonlist.length; i++) {
				var rect:Rectangle = this.buttonlist[i].getAnalyzeRect();
				
				var activePixel:int = 0;
				//Aktivitt im Button berprfen
				for(var ypos:int = 0; ypos < rect.height; ypos++) {
					for(var xpos:int = 0; xpos < rect.width; xpos++) {
						
						if(this.output.getPixel(rect.x + xpos, rect.y + ypos).toString(16) == "ffffff") {
							activePixel++;
						}
					}
				} // Ende activePixel Analyze
				
				var accuracyNow:int = int(Math.round((activePixel / (rect.width * rect.height))*100)); 
				
				// Button Aktiv oder nicht - berprfen welche Ereignisse ausgefhrt werden sollen
				if(accuracyNow >= this.accuracy) {
					if(this.buttonlist[i].getState()) {
						//Wenn die Han schon lnger ber den Button ist
						//this.buttonlist[i].onHandOver();
						this.buttonlist[i].dispatchEvent(new CamButtonEvent(CamButtonEvent.ON_HAND_OVER,true,false));
					} else {
						this.buttonlist[i].setState(true);
						//this.buttonlist[i].onHand();
						this.buttonlist[i].dispatchEvent(new CamButtonEvent(CamButtonEvent.ON_HAND,true,false));
					}
				} else if(this.buttonlist[i].getState()) {
					this.buttonlist[i].setState(false);
					//this.buttonlist[i].onHandOff();
					this.buttonlist[i].dispatchEvent(new CamButtonEvent(CamButtonEvent.ON_HAND_OFF,true,false));
				}
			
			} // ButtonSchleife
		
		}// Ende Funkton
		
	}
	
}
