/************************************************************************************************************
// LightTracking Class ver 2.5 erstellt am 14. Juni 2007
//
// Diese Klasse kann Lichter im Webcam erkennen und Tracken. Als Ergebnis erhaltet ein Bereichsrechteck (Array), in dem sich
// die Lichter befinden.
//
// Author: Florian Weil --- http://www.derhess.de
//
// Die Klasse kann frei fr nicht kommerzielle Zwecke verwendet werden. 
// Bei kommerzieller Verwendung bitte eine kurze Mail an info@derhess.de 
//

/************************************************************************************************************/

package de.derhess.iaCam
{	
	
	import flash.display.BitmapData;
	import flash.filters.ColorMatrixFilter;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	
	import flash.media.Video;
	
	import flash.events.*;
	import de.derhess.iaCam.LightEvent;
	import de.derhess.utils.ColorConverter;
	
	public class LightTracking extends EventDispatcher 
	{
		// Statische Variablen fuer den Modus zum Tracking
		public static const ONE_LIGHT:int = 0;
		public static const MORE_LIGHTS:int = 1;

		// Konfigurationsvariablen
		private var scaleFactor:Number; 
		private var schwellenwert:int; // Ab welchen Helligkeitswert wird getrackt
		private var actionLimit:int; // Gibt an wieviel Durchlufe das Tracking erfloglos sein darf 

		// sonstige Members
		private var webcam:Video;
		private var webcamBitmap:BitmapData;
		private var output:BitmapData;
		private var action:int;  // Toleranz, wie oft hintereinander kein Licht gefunden werden darf
		private var colorCon:ColorConverter;

		//////////////// Speicherung von Ergebnissen ////////////////////
		private var oldResultTracking:Array;  // Speicherung von mehreren LichterTrackings

		// Konstruktor
		function LightTracking(webcamSignal:Video,schwellHelligkeit:int) 
		{
			this.init(webcamSignal,schwellHelligkeit);
		} // Ende Konstruktor
		
		
		// Hilfsfunktion fr Konstruktor - aus Perfomancegrnden
		private function init(webcamSignal:Video,schwellHelligkeit:int):void
		{	
			var standardSize:int = 80; // Variable fr die Bitmapgroesse (in der Breite) des Analysebildes
			
			this.scaleFactor = Math.round(100/(webcamSignal.width  / standardSize));  // Perfomance -> Skalierung der Analysebildes auf 80x60px
			
			this.webcam = webcamSignal;
			this.webcamBitmap =	new BitmapData(this.webcam.width/(100/this.scaleFactor),this.webcam.height/(100/this.scaleFactor),true,0xFF000000);
			this.output =	new BitmapData(this.webcam.width/(100/this.scaleFactor),this.webcam.height/(100/this.scaleFactor),true,0xFF000000);
			
			this.action = 0; // Anfangswert fr Frameanzahl unaktive Lichtquellen
			this.actionLimit = 5;
			
			this.oldResultTracking= new Array();
			// berprfung der Helligkeitsschwellenangabe
			if(schwellHelligkeit > 255) {
				this.schwellenwert = 255;
			} else if (schwellHelligkeit < 0) {
				this.schwellenwert = 0;
			} else {
				this.schwellenwert = schwellHelligkeit;
			}
			
			
			
			// Mach das Bild SW fr die Helligkeitsanalyse --- evtl. spter in der Bildverarbeitung
			var filterMatrix:Array = [0.33,0.33,0.33,0,0.33,
										0.33,0.33,0.33,0,0.33,
										0.33,0.33,0.33,0,0.33,
										1,1,1,1,0.33];
			
			var farbFilter:ColorMatrixFilter = new ColorMatrixFilter(filterMatrix);
			this.webcamBitmap.applyFilter(this.webcamBitmap,this.webcamBitmap.rect,new Point(0,0),farbFilter);
		}

		////////////////// //////////////////////////////////////////////////////////
		//////////// Initialisierung Tracking
		public function startTracking(modus:int):void {
			if(modus == 0) {
				// einLicht Tracking
				this.oneLightTracking();
			} else {
				// Mehrere Lichter tracken
				this.moreLightsTracking();
			}
		}

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// ///////////Tracking fr eine Lichtquelle   //////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		private function oneLightTracking ():void
		{	
			// Schauen ob helle Punkte im Bild vorhanden sind - mindestens 5Pixel 
			var helligkeit:int = (this.webcamBitmap.height * this.webcamBitmap.width) - this.bildOptimierung();
			
			if(helligkeit >= 1) {
				
				var trackSuccess:Boolean = false;
				// Bilder nach Lichter durchsuchen
				for(var y:int = 2; y < this.webcamBitmap.height-2; y = y+2) {
					for(var x:int = 2; x < this.webcamBitmap.width-2;x = x +2) {
					
						//Wenn der Pixel weiss ist dann berprfe die Nachbarn
						if(check_Nachbarn(int(x),(y))) {
							this.oldResultTracking[0] = this.checkLightSize(int(x),int(y));
							this.action = 0;
							trackSuccess = true;
							dispatchEvent(new LightEvent(LightEvent.RESULT_ONE_LIGHT,true,false,this.oldResultTracking[0],null));
						}
						
					} // Ender erste Schleife
				} // Ende 2. Schleife
				
				// Wenn kein Licht gefunden wurde, Setze den Zhler hoch, oder werde ein Event, dass kein Licht mehr vorhanden ist
				if(!trackSuccess) { 
					if(this.action < this.actionLimit) {
						this.action++;
					} else {
						dispatchEvent(new LightEvent(LightEvent.NO_LIGHT_FOUND,true,false));
					}
				}
				
			} else if(this.action < this.actionLimit) {
				this.action++;
				// Wenn kein Licht gefunden wurde, gib erstmal ein leeres Rechteck zurck
				this.oldResultTracking[0] = new Rectangle(0,0,0,0);
				dispatchEvent(new LightEvent(LightEvent.TRACKING_FAILURE,true,false,this.oldResultTracking[0],null));
			}  else {
				// Wenn kein Licht gefunden wurde
				dispatchEvent(new LightEvent(LightEvent.NO_LIGHT_FOUND,true,false));
			}
			
		}

		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// ///////////Tracking fr mehrere Lichtquellen   //////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		private function moreLightsTracking():void
		{
			var neuResultTracking:Array = new Array();
			var resultCounter:int = 0;
			
			// Schauen ob helle Punkte im Bild vorhanden sind - mindestens 5Pixel 
			var helligkeit:int = (this.webcamBitmap.height * this.webcamBitmap.width) - this.bildOptimierung();
			
			if(helligkeit >= 1) {
				for(var y:int = 2; y < this.webcamBitmap.height-2; y = y+2) {
					for(var x:int = 2; x < this.webcamBitmap.width-2;x = x +2) {
						// berprfen ob der Pixel berprft werden soll oder nicht
						var pixelfarbe:uint = this.webcamBitmap.getPixel32(x,y);
						
						var alpha:uint = (uint(pixelfarbe) >> 24) & 0xFF;
						
						if(alpha == 255) {
							// Wenn der Pixel weiss ist dann berprfe die Nachbarn
							if(this.check_Nachbarn(x,y)) {
								neuResultTracking.push(this.checkLightSize(x,y));
							} 
						} // Ende Tracking
					}
				} // Ende Bilddurchlauf
				
				// Wenn schon Lichter vorhanden sind
				if(this.oldResultTracking.length  == neuResultTracking.length && neuResultTracking.length != 0) {
					//////////////////////////////// Sotierung der mehreren Lichtquellen ///////
					// Vergleiche die alten Tracking Ergebnisse mit den Neuen
					var ergebnisTracking:Array = new Array();
					
					for(var i:int = 0; i < neuResultTracking.length; i++) {
						var track_index:int = 0;
						
						var min_distanz:Number = Math.floor(Point.distance(this.oldResultTracking[0].topLeft,neuResultTracking[i].topLeft));
				
						// suche Passendes Rechteck
						for(var k:int = 1; k < this.oldResultTracking.length; k++) {
							
							if(min_distanz > Math.floor(Point.distance(this.oldResultTracking[k].topLeft,neuResultTracking[i].topLeft))) {
								track_index = k;
							}
							
						}
						ergebnisTracking[track_index] = neuResultTracking[i];
					}
					this.oldResultTracking = ergebnisTracking;
					trace("DispatchAnalyse: "+ this.oldResultTracking.length);
					dispatchEvent(new LightEvent(LightEvent.RESULT_MORE_LIGHTS,true,false,ergebnisTracking[0],ergebnisTracking,0));
					
				// Wenn vorher keine Lichter oder neue Lichter vorhanden sind, dann nimm die aktuelle Anzahl
				// ohne deren Beziehung zueinander zu prfen
				} else if(neuResultTracking.length > 0){
					// Ermittlung wieviele Lichter hinzugekommen sind oder verschwunden sind
					var diff:int = this.oldResultTracking.length - neuResultTracking.length
					if(diff > 0) {
						dispatchEvent(new LightEvent(LightEvent.REMOVE_LIGHT,true,false,this.oldResultTracking[0],this.oldResultTracking,diff));
					} else {
						diff = Math.abs(diff);
						dispatchEvent(new LightEvent(LightEvent.ADD_LIGHT,true,false,this.oldResultTracking[0],this.oldResultTracking,diff));
					}
					
					this.oldResultTracking = neuResultTracking;
					dispatchEvent(new LightEvent(LightEvent.RESULT_MORE_LIGHTS,true,false,this.oldResultTracking[0],this.oldResultTracking,diff));
				} else if(this.action < this.actionLimit) {
					this.action++;
					// Wenn kein Licht gefunden wurde
					dispatchEvent(new LightEvent(LightEvent.TRACKING_FAILURE,true,false));
				}  else {
					this.oldResultTracking = new Array();
					// Wenn kein Licht gefunden wurde und ader Aktiv Status berschritten wurde
					dispatchEvent(new LightEvent(LightEvent.NO_LIGHT_FOUND,true,false));
				}
			
			
			}// Ende IF-Block, wenn Lichter vorhanden sind
			else if(this.action < this.actionLimit) {
					this.action++;
					// Wenn kein Licht gefunden wurde, gib erstmal ein leeres Rechteck zurck
					this.oldResultTracking = new Array();
					dispatchEvent(new LightEvent(LightEvent.TRACKING_FAILURE,true,false));
			}  else {
					this.oldResultTracking = new Array();
					// Wenn kein Licht gefunden wurde und ader Aktiv Status berschritten wurde
					dispatchEvent(new LightEvent(LightEvent.NO_LIGHT_FOUND,true,false));
			}
		} // Ende Funktion


		///////////////////////////////////////////////////////////////////////////////////////////////////////////7
		//////////////////// Getter und Setter Funktionen
		public function getBitmapData():BitmapData
		{
			return this.output;
		}

		// Getter Schwellenwert im Bereich 0 - 255
		public function getThreshold():int
		{
			return this.schwellenwert;
		}

		// Setter Schwellenwert, aber welcher Helligkeit getrackt werden soll
		public function setThreshold(schwellHelligkeit:int):void 
		{
			if(schwellHelligkeit > 255) {
				this.schwellenwert = 255;
			} else if (schwellHelligkeit < 0) {
				this.schwellenwert = 0;
			} else {
				this.schwellenwert = schwellHelligkeit;
			}
		}


		//////////////////////////////////////////////////////////////////////////
		//////////////// Hilfsfunktionen

		// bereitet das Webcambild fr die Analyse auf - erstellt das Binrbild
		private function bildOptimierung():int
		{
			
			var scaleMatrix:Matrix = new Matrix();
			scaleMatrix.scale(this.scaleFactor/100,this.scaleFactor/100);

			this.webcamBitmap.draw(this.webcam,scaleMatrix);
			
			var HexSchwellenwert:Number = Number("0xFF"+ this.schwellenwert.toString(16) + this.schwellenwert.toString(16) + this.schwellenwert.toString(16));
			var unaktivfarbe:Number = 0xFF000000; // schwarz
			var aktivfarbe:Number = 0xFF00FF00;  // weiss
			
			var changePixel:int = this.webcamBitmap.threshold(this.webcamBitmap,this.webcamBitmap.rect,new Point(0,0),"<=",Number(HexSchwellenwert),unaktivfarbe,aktivfarbe,true);
			this.output.copyPixels(this.webcamBitmap, this.webcamBitmap.rect,new Point(0,0));
			
			return changePixel;
		}

		// Suche nach mehreren weissen Pixeln unter Beachtung eines bestimmten Rasters -- Perfomancelastig --
		private function check_Nachbarn (x:int,y:int):Boolean 
		{
			var counter:int = 0;
			// oben Mitte
			//if(uint(this.webcamBitmap.getPixel(x,y-1).toString(16)) != 0x000000) counter++;
			// rechts oben
			if(uint(this.webcamBitmap.getPixel(x+1,y-1).toString(16)) != 0x000000) counter++;
			// rechts Mitte
			if(uint(this.webcamBitmap.getPixel(x+1,y).toString(16)) != 0x000000) counter++;
			// rechts unten
			if(uint(this.webcamBitmap.getPixel(x+1,y+1).toString(16)) != 0x000000) counter++;
			// unten Mitte
			//if(uint(this.webcamBitmap.getPixel(x,y+1).toString(16)) != 0x000000) counter++;
			
			if(counter == 3) {
				return true;
			} else {
				return false;
			}
		}


		// Wenn ein Licht gefunden wurde, dann ermittle die Gre des Lichtes
		private function checkLightSize(x:int,y:int):Rectangle
		{
			var x_track:int = 0;
			var y_track:int = 0;
			var width_track:int = 1;
			var height_track:int = 1;
			
			x_track = x;
			y_track = y;
			
			// Scanne die Breite des Lichtes
			
			// Scanne die Hoehe des Lichtes funktioniert ganz gut
			while(this.check_Nachbarn(x,int(y+height_track))) { height_track++; }
			// Scanne obere Breite
			while(this.check_Nachbarn(int(x + width_track),y)) { width_track++; }
			// Scanne die Breite des Lichtes, aber in deren mittleren Hhe
			y  = int(Math.floor(y + (height_track / 2)));
			var mittlerebreite:int= 1;
			while(uint(this.webcamBitmap.getPixel(x + mittlerebreite,y).toString(16)) != 0x000000) { mittlerebreite++; }
			y = y_track; // Wiederherstellung der alten Koordinate, damit die Schleife noch richtig arbeiten kann
			
			
			// zu kleine Rechteck Abweichung ausgleichen - hauptschlich in der Breite gab es Probleme
			var offset:int = Math.abs(mittlerebreite - width_track);
			x_track -= int(Math.floor(offset/2)); // Standardwert ist einfach 2
			y_track -= 2; // Standardwert ist einfach 2
			width_track += (offset); // Standardwert ist einfach width_track += 4 oder (offset*2)
			height_track += 4; // Standardwert ist einfach 4
			
			// Ausgabe Rechteck Grsse
			
			this.webcamBitmap.fillRect(new Rectangle(x_track - 4,y_track - 4,width_track+8,height_track+8),0x00000000);
			// Speichern des Ergebnisses
			var scale:Number = 100 /this.scaleFactor;
			var hotpoint:Rectangle = new Rectangle(x_track*scale,y_track*scale,width_track*scale,height_track*scale);
			return hotpoint;
		}

	}
}
