/**
* ColorConverter Class
*
* Die ColorConverter Hilfsklasse rechnet Farbwerte in das Hexadezimal System um, oder wieder zurck.
* Darber hinaus kann man noch seperate Farbwerte umndern.
* 
* @author Florian Weil / Homepage: www.derhess.de
* @version 0.5
* 
*
* Klasse basiert hauptschlich auf der OnlineTechNote von Colin Moock
* zum Nachschlagen auf http://www.moock.org/asdg/technotes/bitwise
*
* TO DO LISTE:
* 
*
*/

package de.derhess.utils
{	
	/**
	 *  The ColorConverter Class converts Hexadecimal color values to decimal values and decimal 
	 *  color values to Hexadecimal color values
	 *
	 */  
	public class ColorConverter
	{
		private var colorValue:uint;
		
		public function ColorConverter (color:uint) {
			colorValue = color;
		}
		
		 /**
		 *  The static function <code>Decimal2Hex()</code> converts a decimal ARGB Color Value to a He
		 *  a hexadecimal color value.
		 *
		 *  @param alpha alpha channel value. Range must be 0-255.
		 *
		 *  @param red red channel value. Range must be 0-255.
		 *
		 *  @param green green channel value. Range must be 0-255.
		 *
		 *  @param blue blue channel value. Range must be 0-255.
		 *
		 *  @return uint the converted hexadecimal color value.
		 */  
		// Konvertiert Dezimal Farben ins Hexadezimal System
		public static function Decimal2Hex(alpha:uint, red:uint, green:uint, blue:uint):uint
		{
			return alpha<<24 | red<<16 | green<<8 | blue;;
		}
		
		
		/**
		 *  The static function <code>Hex2Decimal()</code> converts a hexadecimal color value to a
		 *  decimal ARGB Color Value. The Value is represented in a Object.
		 *
		 *  @param color Hexadecimal color value in 32bit.
		 *
		 *  @return Object the converted hexadecimal color value to ColorObject. The Object Members are
		 * 	alpha = alpha channel value in decimal
		 * 	red = red channel value in decimal
		 * 	green = green channel value in decimal
		 * 	blue = blue channel value in decimal
		 */  
		// Hexadezimal Farbangaben werden in die verschiedenen Kanaele aufgesplittet
		public static function Hex2Decimal(color:uint):Object
		{
			var colorInformation:Object = new Object();
			colorInformation.alpha = (color >> 24) & 0xFF;
			colorInformation.red = (color >> 16) & 0xFF;
			colorInformation.green = (color >> 8) & 0xFF;
			colorInformation.blue = color & 0xFF;
			
			return colorInformation;
		}
		
		///////////////////////////////////////////////////////////////////////////////////
		// SETTER METHODEN
		///////////////////////////////////////////////////////////////////////////////////
		
		// neuen Farbwert setzen
		public function setColor(color:uint):void
		{
			colorValue = color;
		}
		
		// neuen Farbwert setzen
		public function setARGBColor(alpha:uint, red:uint, green:uint, blue:uint):void
		{
			colorValue = alpha<<24 | red<<16 | green<<8 | blue;
		}
		
		// neuen Farbwert setzen
		public function setRGBColor(red:uint, green:uint, blue:uint):void
		{
			var alpha:uint = 255;
			colorValue = alpha<<24 | red<<16 | green<<8 | blue;
		}
		
		// neuen Alpha Kanal ndern
		public function setAlphaChannel(value:int):void
		{
			if(value < 0 || value > 255) {
				throw new RangeError("Value must be in the range 0-255.");
			}
			colorValue &= (0x00FFFFFF);
			colorValue |= (value<<24);
		}
		
		// neuen Rot Kanal ndern
		public function setRedChannel(value:int):void
		{
			if(value < 0 || value > 255) {
				throw new RangeError("Value must be in the range 0-255.");
			}
			colorValue &= (0xFF00FFFF);
			colorValue |= (value<<16);
		}
		
		// neuen Grn Kanal ndern
		public function setGreenChannel(value:int):void
		{
			if(value < 0 || value > 255) {
				throw new RangeError("Value must be in the range 0-255.");
			}
			colorValue &= (0xFFFF00FF);
			colorValue |= (value<<8);
		}
		
		// neuen Blau Kanal ndern
		public function setBlueChannel(value:int):void
		{
			if(value < 0 || value > 255) {
				throw new RangeError("Value must be in the range 0-255.");
			}
			colorValue &= (0xFFFFFF00);
			colorValue |= (value);
		}
		
		///////////////////////////////////////////////////////////////////////////////
		// GETTER METHODEN
		///////////////////////////////////////////////////////////////////////////////
		
		// Gib Farbwert zurck
		public function getColor():uint
		{
			return colorValue;
		}
		
		// Konvertiert Dezimal Farben ins Hexadezimal System
		public function getRGBColor():uint
		{
			setAlphaChannel(0);
			return colorValue;
		}
		
		// Hexadezimal Farbangaben werden in die verschiedenen Kanaele aufgesplittet
		public function getARGBColor():Object
		{
			var colorInformation:Object = new Object();
			colorInformation.alpha = (colorValue >> 24) & 0xFF;
			colorInformation.red = (colorValue >> 16) & 0xFF;
			colorInformation.green = (colorValue >> 8) & 0xFF;
			colorInformation.blue = colorValue & 0xFF;
			
			return colorInformation;
		}
		
		// Gib Alpha Kanalwert zurck
		public function getAlphaChannel():int
		{
			return (colorValue >> 24) & 0xFF;
		}
		
		// Gib Rot Kanal zurck
		public function getRedChannel():int
		{
			return (colorValue >> 16) & 0xFF;
		}
		
		// Gib Grn Kanal zurck
		public function getGreenChannel():int
		{
			return (colorValue >> 8) & 0xFF;
		}
		
		// Gib Blau Kanal zurck
		public function getBlueChannel():int
		{
			return (colorValue >> 16) & 0xFF;
		}
		
	}
}