/**
* MathExtension Class
*
* Die MathExtension Klasse soll weitere mathematische Funktionalitt anbieten, die die Standard Math Klasse
* nicht liefert. Zu den Aufgaben gehren Defintionsbereiche erkennen, Prozentrechnung, Grad- und Bogenmaumrechnung
*
* 
* @author Florian Weil - www.derhess.de
* @version 0.1
*/

package de.derhess.utils
{	 

	public class MathExtension {
		
		// Konstruktor
		public function MathExtension() { };
		
		 /**
		 *  The static function <code>rangeCheck()</code> checks a number is in a range between number a and number b
		 *
		 *  @param a minimum of the range.
		 *
		 *  @param b the maximum of the range.
		 *
		 *  @param c the checking number
		 *
		 *  @return boolean if the number in the range then the function returns true. If not the return value is false
		 */  
		public static function rangeCheck(a:Number, b:Number, c:Number):Boolean
		{
			if(a <= c && c <= b) {
				return true;
			} else {
				return false;
			}
		}
		
		 /**
		 *  The static function <code>getDifference()</code> returns the difference between two numbers
		 *  Die Funktion gibt die Differenz zweier Zahlen zurck
		 * 
		 *  @param a number one / Erste Zahl
		 *
		 *  @param b number two / Zweite Zahl
		 *
		 *  @return number return value is the always the positive difference value between two numbers 
		 * Gibt die Differenz zweier Zahlen immer im positven Wert zurck
		 */  
		public static function getDifference(a:Number, b:Number):Number
		{
			return Math.abs(a-(b));
		}
		
		/**
		 *  The static function <code>percentage()</code> returns the value in percent
		 *  Gibt den Prozentsatz zurck
		 *
		 *  @param percentValue the value, which percentage you want to know / der Prozentwert
		 * 
		 *  @param base the maximum value / Grundwert, der Wert fr 100%
		 *
		 *  @return int the return value in percent 0-100 / Prozentsatz in 0-100
		 */  
		public static function percentage(percentValue:Number,base:Number):int
		{	
			return Math.floor((percentValue/base)*100);
		}
		
		
		/**
		 *  The static function <code>percentValue()</code> returns the percent value
		 *  Gibt den Prozentwert zurck
		 *
		 *  @param percentage a value between 0-100 / gewnschte Prozentangabe - Prozentsatz
		 * 
		 *  @param base the maximum value / Grundwert, der Wert fr 100%
		 *
		 *  @return int the returns the percentValue / Prozentwert
		 */  
		public static function percentValue(percentage:int,base:Number):int
		{	
			return Math.floor((percentage/100)*base);
		}
		
		/**
		 *  The static function <code>percentBase()</code> returns the base of the percentage
		 *  Gibt den Prozentwert zurck
		 *
		 *  @param percentage a value between 0-100 / Prozentsatz von 0 - 100%
		 * 
		 *  @param percentValue the value of the percentage / Prozentwert
		 *
		 *  @return number the returns the base value / Grundwert
		 */  
		public static function percentBase(percentage:Number,percentValue:Number):Number
		{	
			return Math.floor((percentValue/percentage)*100);
		}
		
		
		/**
		 *  The static function <code>getRad()</code> returns the radiant value
		 *  Gibt die Gradzahl im Bogenma zurck
		 *
		 *  @param degree a value in a degree unit / Ein Wert im Gradma
		 *
		 *  @return number returns the radiant / Umrechnungswert im Bogenma
		 */  
		public static function getRad(degree:uint):Number
		{	
			return degree*(Math.PI/180);
		}
		
		
		/**
		 *  The static function <code>getDegree()</code> returns the degree value
		 *  Gibt die Gradzahl im Bogenma zurck
		 *
		 *  @param radiant a value in a radiant unit / Ein Wert im Bogenma
		 *
		 *  @return number returns the degree value / Umrechnungswert im Gradma
		 */  
		public static function getDegree(radiant:Number):Number
		{	
			return radiant*(180/Math.PI);
		}
	}
	
}